//*********************************************************************//
//Function to receive data in Manchester Code, according to IEEE 802.3 //
//definition ("1" is "LtoH" and "0" is "HtoL").                        //
//This function ("mc_rx") is called from the main program and it       //
//returns a 32-bit value containing 4 bytes of information: ADDRESS,   //
//DATA, CRC and SEMIPERIOD (half bit time) of the received frame.      //
//The function calls the different sub-functions in order.             //
//Function "baud_detect" measures the length of the first received bit //
//and determines the transmision speed, basically by reading the timer0//
//information. By a simple calculation, using a 20MHz clock, and       //
//setting the timer0 to be divided by 16 (in the main program), a      //
//transmision speed of 5000 bps (period = 200 microseconds or us) gives//
//a timer0 count = 62.5 (1/(20MHz/4) = 0.2us; 16 x 0.2us = 3.2us, so   //
//timer0 increments every 3.2us; thus 200us/3.2us = 62.5us). Since the //
//received signal may experiment "jitter" (due to the nature of the    //
//radio transmission), the pulse may appear slightly shorter or longer,//
//so a margin is given for the detection (55 to 70). The same applies  //
//to a 2500bps transmission (400us period), margin is 118 to 133 (125).//
//The SEMIPERIOD is detected and later used in all other bit-detection //
//functions. The next function is "sync_detect", which counts the      //
//remaining 18 "1" of the frame synchronism and the final "0". If an   //
//error is detected in any of these or the next functions, it is stated//
//by setting the variable err_rx = 1. The function "failsafe" prevents //
//the program to enter in an infinite wait state in case the exit      //
//condition of a loop is not met due to errors in the received signal. //
//If the variable timer_rx exceeds the semiperiod, then an error flag  //
//is set and the program exits the loop. Finally, the remaining 3 bytes//
//are received and stored in a 32-byte variable (three_byte_rx), by    //
//feeding the received bits sequentially, and eliminating the "1" and  //
//"0" between bytes by the function called "spacer". If there was no   //
//error, the 3 received bytes are stored in frame_rx (32-bit), moved 1 //
//byte up (x 256) and SEMIPERIOD byte added as the less significant    //
//byte. This 32-bit value is then returned to the main program.        //
//If an error occurs, frame_rx is loaded with the value 0, so the main //
//program "understands" that there is no valid transmission present (or//
//the reception was incomplete) and in this case the variables must not//
//be updated, or whatever other action done to acknowledge the error.  //
//Microcontroller Xtal: 20MHz - No Watch Dog Timer - No Power Up Timer.//
//*********************************************************************//

//Definitions
short err_rx=0;
int semiperiod;
long timer_rx;
int t_rx;
int jitter;
long semiandjitter;
int bitcount;
int bytecount;
int in1;
int in2;
short bit_rx;
int32 three_byte_rx;
int semi;
int32 ad_da_cr_rx;
//End of definitions

int baud_detect(int semi)
{
            timer_rx=0;
            //The mc_rx function is called after a LOW to HIGH transition,
            //so Pin B0 must be high; the program will wait until this
            //condition is cleared (Pin B0 returns to 0).
            while(input(PIN_B0)==1)
            {
               delay_us(1);
               timer_rx=timer_rx+1;
               if(timer_rx>=1000)      //This code guarantees that the
               {                       //program will not stay here
                  err_rx=1;            //forever in case there is an
                  goto err_bau;        //error; when "timer_rx" reaches
               }                       //1000, the error flag is set and
            }                          //the function exits.
            timer_rx=0;
            //After Pin B0 returns to 0, the function reaches this step;
            //it will stay here until Pin B0 returns to 1 again.
            while(input(PIN_B0)==0)
            {
               delay_us(1);
               timer_rx=timer_rx+1;    //Infinite loop protection code.
               if(timer_rx>=1000)
               {
                  err_rx=1;
                  goto err_bau;
               }
            }
            //Here the function reaches the line which is its true
            //purpose: measuring the length of the first bit received.
            //By waiting a high state (1), followed by a low state (0),
            //the function waited for a whole bit, while timer0 was
            //counting. Now, timer0 will have the duration of 1 bit,
            //thus the bit rate or, in other words, the timing information.
            //This is only valid if we are in the synchronism part of the
            //frame, with several successive "1"; a "1" then "0" sequence
            //will give a wrong bit rate.
            t_rx=get_timer0();
            
            if((t_rx>55)&&(t_rx<70))   //The result of timer0 is converted
            semi=100;                  //into time in actual microseconds,
            if((t_rx>118)&&(t_rx<133)) //and put into "semi" (half period).
            semi=200;                  //The range compensates for jitter.
                        
            timer_rx=0;
            //Another half bit is read here, to complete 2 full bits.
            //According to the IEEE 802.3 definition, a "1" is represented
            //by a "low to high" transition. The RF link idle state is low,
            //so the interrupt is triggered in the middle of the first "1"
            //of the frame. By waiting a high, then low states to calculate
            //the bit rate, we use the second half of the first bit and the
            //first half of the second; thus we wait another half to
            //complete 2 full bits, hopefully the first 2 of a valid frame.
            while(input(PIN_B0)==1)
            {
               delay_us(1);
               timer_rx=timer_rx+1;
               if(timer_rx>=1000)
               {
                  err_rx=1;
                  goto err_bau;
               }
            }

err_bau:   return semi;                //Bit rate information is returned.
}

short failsafe(err_rx)
{
            //Once the semiperiod is obtained, a more precise "infinite
            //loop protection code" can be written. Now, if the variable
            //"timer_rx" reaches "semiperiod", it means that there is an
            //error, so this flag is set and the calling function will
            //exit.
            delay_us(1);
            timer_rx=timer_rx+1;
            if(timer_rx>=semiperiod)
            err_rx=1;
            return err_rx;
}

void sync_detect()
{
            //During bit rate detection, two bits ("1") were received.
            //If we are receiving a valid frame, then we need to expect
            //18 more "1" and one "0". This function does that; if
            //successful, it means that a valid frame synchronism has
            //been detected, so the rest of the data will be valid too.
            bitcount=1;
sync:       in1=input(PIN_B0);
            timer_rx=0;
            //The function waits until the input changes; this could be
            //a low or high state, depending if the received bit is a 
            //"1" (LtoH) or "0" (HtoL) respectively.
lock_in:    if(in1==input(PIN_B0))
            {
               failsafe(err_rx);       //Infinite loop protection code.
               if(err_rx==1)
               goto err_syn;
               goto lock_in;
            }   
            in2=input(PIN_B0);         //After the change, the input is
            bitcount=bitcount+1;       //read, and the bit count is
            if(bitcount>=21)           //increased; if 21 is reached (we
            {                          //were expecting 19 increments of
               err_rx=1;               //"bitcount", 18 "1" and one "0")
               goto err_syn;           //there is an error, and the
            }                          //function exits.
            
            //If no error, it waits for a time equal to half period plus
            //jitter, to make sure it skips the second half of the bit.
            //The state of the second half (previously saved in "in2")
            //is evaluated; if high, it means that the bit is "1", so we
            //are still in the synchronism and the reading loop continues.
            //If low, then the bit is a "0", so it is the end of the
            //synchronism. "bitcount" is evaluated and, if different than
            //20, the error flag is set and the function exits.
            delay_us(semiandjitter);   
            if(in2==1)                 
            goto sync;                   
            if(bitcount!=20) 
            {
               err_rx=1;
               goto err_syn;
            }
err_syn:    return;
}

void spacer()
{
            //There is a "1" followed by a "0" at the end of each byte.
            //This function just reads this "2 bit spacer", disregarding
            //the contents of the bits.
            for(bitcount=1;bitcount<=2;++bitcount)          
            {                          
               timer_rx=0;
               in1=input(PIN_B0);
lock_sp:       if(in1==input(PIN_B0))    
               {
                  failsafe(err_rx);    //Infinite loop protection code.
                  if(err_rx==1)
                  goto err_spa;
                  goto lock_sp;        
               }
               delay_us(semiandjitter);
            }
err_spa:    return;
}

int32 detect(int32 three_byte_rx)                      
{            
            //This is the function that reads the "information" bytes,
            //ADDRESS, DATA and CRC. These 3 bytes are stored into a
            //32-bit variable ("three_byte_rx"), loading the received
            //bits from the LSB end. Every 8-bits, the "spacer" is 
            //discarded, and the next byte is read.
            three_byte_rx=0; 
            for(bytecount=1;bytecount<=3;++bytecount)
            {
               for(bitcount=1;bitcount<=8;++bitcount)          
               {                          
                  three_byte_rx<<=1;   //Left shift, 1 bit.                        
                  timer_rx=0;                   
                  in1=input(PIN_B0);        
lock_de:          if(in1==input(PIN_B0))    
                  {
                     failsafe(err_rx); //Infinite loop protection code.
                     if(err_rx==1)
                     goto err_det;
                     goto lock_de;        
                  }
                  in2=input(PIN_B0);        
                  if(in1>in2) bit_rx=0;//If 1st half higher than 2nd.          
                  if(in1<in2) bit_rx=1;//If 1st half lower than 2nd.         
                  three_byte_rx=three_byte_rx+bit_rx;               
                  delay_us(semiandjitter);          
               }
               spacer();
            }
err_det:    return three_byte_rx;                  
}                                   

int32 mc_rx(int32 frame_rx)
{
            //This is the key function called from the main program.
            //The actual "bit-gathering" functions are called in order,
            //and the result is SEMIPERIOD (half bit time) and a 32-bit
            //variable ("ad_da_cr_rx") containing the ADDRESS, DATA and
            //CRC. This variable is copied into "frame_rx" (also 32-bit)
            //moved 1 byte up and SEMIPERIOD added. The result is that
            //"frame_rx" contains all 4 relevant bytes, and it is
            //returned to the main program if a valid frame has been
            //received. If the interrupt was called by an erratic pulse,
            //the error is indicated by returning an empty "frame_rx".
            set_timer0(0);
            err_rx=0;
         
            semiperiod=baud_detect(semi);
            if(err_rx==1)
            goto error;
            jitter=semiperiod/2;
            semiandjitter=(long)semiperiod+jitter;
         
            sync_detect();
            if(err_rx==1)
            goto error;
         
            ad_da_cr_rx=detect(three_byte_rx);
            if(err_rx==1)
            goto error;
         
error:      if(err_rx==0)
            {
               frame_rx=ad_da_cr_rx;
               frame_rx=frame_rx*256+semiperiod;
            }
            else
            frame_rx=0;
            return frame_rx;
}
