#include "Manchester_Link_RX.h"
#include "MAN_RX.c"

//********************************************************************//
//Main program; here is where the user programs the microcontroller to//
//do whatever is intended with the received information, captured by  //
//the "mc_rx" function (specified in the #include <MAN_RX.c>).        //
//The "main" function sets the initial conditions first, a couple of  //
//them are worth mentioning: "output_low(PIN_A3);" to be sure the LED //
//is OFF; "setup_timer_0(RTCC_INTERNAL|RTCC_DIV_16|RTCC_8_bit);" to   //
//program the "timer0" so it is 8-bit long and increments in 1/16th of//
//the instruction clock frequency. The other important instruction is //
//intended to enable all interrupts: "enable_interrupts(GLOBAL);",    //
//provided the individual flag is set. These 3 lines must remain.     //
//Then the program enters an infinite loop, waiting for a proper pulse//
//to appear on Port B, bit 0 ("enable_interrupts(INT_EXT);"). This is //
//triggered by a low to high transition ("ext_int_edge(L_TO_H);").    //
//Other tasks could be performed in this loop, but for this example,  //
//the key activities will be performed within the interrupt service   //
//routine (ISR).                                                      //
//When an interrupt is detected, the program goes to the ISR. Here is //
//where the data reception function is called ("frame=mc_rx(frame);"),//
//and the ADDRESS, DATA, CRC and HALFTIME are returned inside "frame".//
//This is accomplished by defining "frame" as a 32-bit variable, thus //
//capable of holding 4 bytes inside, which are those 4 variables. Each//
//variable is extracted in a very simple way; by making the variable  //
//equal to "frame", it will only keep the least significant byte      //
//(since each variable is defined as "int"). Then "frame" is shifted  //
//right 8 times (1 byte) and the next variable is extracted.          //
//The bit rate is defined by a simple "if" statement (avoiding math   //
//calculations to preserve memory) and the error status set. Any      //
//HALFTIME different from the 2 expected values (100 and 200) will    //
//indicate an error. The "mc_rx" function returns 0 is an error is    //
//detected. In case there is no error and ADDRESS has the expected    //
//value (15 in this example), the microcontroller performs the        //
//programmed activities, in this case, powering the LED based on DATA //
//contents: 0 = OFF, 1 = ON.                                          //
//Check the MAN_RX.c file to learn about its variables, since they    //
//should be reserved and the same names not used in this main program.//
//This example uses the PIC16F628A @ 20MHz, and PIN B0(pin 6) to      //
//receive the data from the RX module (this is fixed in hardware).    //
//This demonstrates how to program the RX unit and receive data from  //
//the RX function. For the purpose of this example, ADDRESS is fixed  //
//and CRC = DATA, as the simplest form of data accuracy validation.   //
//The user can use any possible combination, with the only restriction//
//that these 3 variables must be 1 byte long each; if a longer data   //
//must be transmitted, this can be done in succesive transmissions or //
//assigning the high byte to DATA and the low byte to CRC.            //
//Microcontroller Xtal: 20MHz - No Watch Dog Timer - No Power Up Timer//
//********************************************************************//

//Definitions
int32 frame;
int halftime;
int address;
int data;
int crc;
short error;
long bitrate;
void detection_isr();
//End of definitions

void main()
{
   set_tris_a(0x00);
   output_low(PIN_A3);           //LED is OFF.
   //Next line programs the "timer0" so it is 8-bit long and increments
   //in 1/16th of the instruction clock frequency.
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_16|RTCC_8_bit);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   enable_interrupts(GLOBAL);    //All interrupts enabled.
     
   While(1)                      //Infinite loop.
   {
      enable_interrupts(INT_EXT);
   
      ext_int_edge(L_TO_H);

//The user may do more activities here, the only restriction is that
//the interrupts must be enabled; some activities, like showing data
//on a character LCD may disable interrupts, so these kind of jobs
//must be programmed inside the ISR, after the information is
//received and validated.
   
   }
}

#int_ext
void detection_isr()
{
   disable_interrupts(INT_EXT);  //Disabled to prevent re-entrance.
   frame=mc_rx(frame);           //32-bit "frame" contains ADDRESS as
                                 //the most significant byte, then
   halftime=frame;               //DATA, CRC and HALFTIME.
   frame>>=8;                    //"frame" is shifted right 1 byte.
   crc=frame;
   frame>>=8;
   data=frame;
   frame>>=8;
   address=frame;
            
   error=0;
   if(halftime==100)             //The bit rate is set based on 
   bitrate=5000;                 //HALFTIME contents.
   else if(halftime==200)                       
   bitrate=2500;
   else                          //If none of the expected values is
   error=1;                      //received, error flag is set.
               
   if((address==15)&&(error==0)) //If no error and ADDRESS equal the
   {                             //set value (15), then the status of
      if(data==0)                //the LED is changed based on DATA
      output_low(PIN_A3);        //contents: 0 means OFF, while 1
      if(data==1)                //means ON.
      output_high(PIN_A3);
   }
   else                          //If DATA contains any other value,
   output_low(PIN_A3);           //the LED will be OFF.
   
//Here the user is free to add code, which will be inside the ISR,
//meaning that the INT_EXT interrupt is still disabled. This is
//useful to call routines that may disable interrupts themselves,
//since this will not affect the normal data reception, as mentioned.
//If the intention is to show the information on an LCD, here is
//where to put the LCD service routine.

}
